"""
To use, either use Plone 2.0+ to install, or add an external 
method to the root of the Plone Site that you want the skin registered 
in with the configuration:

 id: install_skin
 title: Install Skin *optional*
 module name: PloneJUpload.Install
 function name: install

Then go to the management screen for the newly added external method
and click the 'Try it' tab.  
"""

from Products.CMFCore.TypesTool import ContentFactoryMetadata
from Products.CMFCore.DirectoryView import addDirectoryViews
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.CMFCorePermissions import ManagePortal

from cStringIO import StringIO
import string

from Products.PloneJUpload import skin_globals

skin_names = ('PloneJUpload',)

def setupSkin(self, out):
    skinsTool = getToolByName(self, 'portal_skins')
    
    # Add directory views
    try:  
        addDirectoryViews(skinsTool, 'skins', skin_globals)
        out.write( "Added directory views to portal_skins.\n" )
    except:
        out.write( '*** Unable to add directory views to portal_skins.\n')

    # Go through the skin configurations and insert the skin
    skins = skinsTool.getSkinSelections()
    for skin in skins:
        path = skinsTool.getSkinPath(skin)
        path = map(string.strip, string.split(path,','))
        changed = 0
        for skin_name in skin_names:
            if skin_name not in path:
                try: 
                    path.insert(path.index('custom')+1, skin_name)
                    changed = 1
                except ValueError:
                    path.append(skin_name)
                    changed = 1

        if changed:        
            path = string.join(path, ', ')
            # addSkinSelection will replace existing skins as well.
            skinsTool.addSkinSelection(skin, path)
            out.write("Added %s to %s skin\n" % (', '.join(skin_names),skin))
        else:
            out.write("Skipping %s skin, %s already set up\n" % (skin, ', '.join(skin_names)))


def addAction(self, out):
    '''Add an upload view to the portal_actions
    '''
    atool = getToolByName(self, 'portal_actions')
    
    atool.addAction( id='PloneJupload',
                     name='Upload Files',
                     action='string:${folder_url}/PloneJUpload_view',
                     condition='',
                     permission='Add portal content',
                     category='folder',
                     visible=1 )
    out.write( "Added action to portal_actions.\n" )
        

def removeAction(self, out):
    ''' Remove the upload view from portal_actions
    '''
    atool = getToolByName(self, 'portal_actions')
    
    actions = atool.listActions()
    # Get the index of our action(s)
    our_actions = [n for n in range(len(actions)) if actions[n].getId()=='PloneJupload']

    if len(our_actions) > 1:
        out.write( "*** More than one PloneJupload action defined, actions not removed.\n" )
        return

    if not our_actions:
        out.write( "*** No PloneJupload action defined.\n" )
        return
    
    try:
        atool.deleteActions(selections=our_actions)
        out.write( "Removed action from portal_actions.\n" )
    except:
        out.write( "*** Unable to remove action from portal_actions.\n" )


def install(self):
    out=StringIO()
    setupSkin(self, out)
    addAction(self, out)
    out.write('Installation of "PloneJUpload" completed.\n')
    return out.getvalue()


def uninstall(self):
    "Just remove skin names from layers"
    out=StringIO()
    skinsTool = getToolByName(self, 'portal_skins')
    skins = skinsTool.getSkinSelections()
    for skin in skins:
        path = skinsTool.getSkinPath(skin)
        path = map(string.strip, string.split(path,','))
        path = ','.join([n for n in path if not n in skin_names])
        skinsTool.addSkinSelection(skin, path)
    out.write('Uninstalled from skin layers succesfully\n')
    removeAction(self, out)
    return out.getvalue()
